- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository: {{ test_definitions }}
{%if device.name != 'ssh-device' %}
      lava-signal: kmsg
{%endif%}
      from: url
      compression: zstd
      path: automated/linux/systemd-analyze/systemd-analyze.yaml
      name: {{ name }}

